#ifndef ITEMS_HPP
#define ITEMS_HPP

/*

This file defines classes that are used to represent various data searched by the server.
It is a simple class hierarchy with a common abstract parent.

*/

#include <string>
#include <set>
#include <vector>
#include <optional>

namespace library {

// Base class for various types of items
class Item{
public:
    virtual ~Item(){};
    virtual const char* typeName() const = 0;
};

// Different types of items.
class Book: public Item{
public:
    // Note: do not change the types of the fields.
    // The types were chosen
    // in order to make you learn how to find a proper counterpart for the Thrift IDL type
    // and how to work with those types in different languages
    std::string name;
    int16_t length;
    std::optional<int32_t> available;

    virtual const char* typeName() const {
        return "Book";
    }
};
class CD: public Item{
public:
    std::vector<std::string> name;
    std::optional<std::vector<int16_t> > length;
    
    virtual const char* typeName() const {
        return "CD";
    }
};
class Magazine: public Item{
public:
    std::set<std::string> name;
    bool available;

    virtual const char* typeName() const {
        return "Magazine";
    }
};

}
#endif