#include "items.hpp"
#include "database.hpp"
#include "string_conversions.hpp"
#include "summary.hpp"

#include <iostream>
#include <vector>

int main(int argc, char** argv){

    // Create a random database of 30 library items
    std::random_device random_device;
    std::mt19937 random_engine(random_device());
    Database database(30, random_engine);

    // Example: find first 5 books.
    {
        std::cout << "Searching for first 5 books" << std::endl;

        std::vector<library::Item*> foundItems;
        database.search(std::set{std::string("Book")}, foundItems, 0, 5);

        for(library::Item* i: foundItems) {
            // We searched for books, so we can cast the item to library::Book
            if(library::Book* book = dynamic_cast<library::Book*>(i)){
                std::cout << "Found book: " << book -> name << std::endl;
            }
            else {
                std::cout << "Error: unexpected type of item" << std::endl;
            }
        }
    }

    // Example: summary of all items
    {
        std::cout << std::endl << "Summary of all items" << std::endl;

        // Here we build the summary
        SummaryBuilder summaryBuilder;

        // Example of a query, done in batches of 3
        const int N = 3;

        // The index is used to store a current position in the database across multiple calls to database.search
        int index = 0;
        for(;;){
            // Here we store a batch of items
            std::vector<library::Item*> foundItems;
            // Query the database for the next N items
            index = database.search(std::set{std::string("Book"), std::string("CD"), std::string("Magazine")}, foundItems, index, N);
            // Repeat while we find some items
            if(foundItems.empty())
                break;

            // Add the items to the summary
            for(library::Item* i: foundItems) {

                if(library::Book* book = dynamic_cast<library::Book*>(i)){
                    summaryBuilder.add(*book);
                }
                else if(library::CD* cd = dynamic_cast<library::CD*>(i)){
                    summaryBuilder.add(*cd);
                }
                else if(library::Magazine* magazine = dynamic_cast<library::Magazine*>(i)){
                    summaryBuilder.add(*magazine);
                }
                else {
                    std::cout << "Error: unexpected type of item" << std::endl;
                }
            }
        }

        // Display the summary
        for(const auto& entry: summaryBuilder.get()) {
            std::cout << entry.first << ": " << to_string_commas(entry.second) << std::endl;
        }
    }
}