#ifndef SUMMARY_HPP
#define SUMMARY_HPP

#include <string>
#include <map>

#include "items.hpp"
#include "string_conversions.hpp"

/*

Generates a summary required by the Reports service.
Use the add methods to add the items to the report.
Then use get to get a map from field names to sets of values.

*/

class SummaryBuilder{
    std::map<std::string, std::set<std::string>> summary;
public:
    void add(const library::Book& book){
        // std::string name
        summary["name"].insert(book.name);
        // int16_t length
        summary["length"].insert(std::to_string(book.length));
        // std::optional<int32_t> available
        if(book.available)
            summary["available"].insert(std::to_string(*book.available));
    }
    void add(const library::CD& cd){
        //  std::vector<std::string> name
        summary["name"].insert(to_string_commas(cd.name));

        if(cd.length)
            summary["length"].insert(to_string_commas(*cd.length));
        
    }
    void add(const library::Magazine& magazine){
        summary["name"].insert(to_string_commas(magazine.name));
        summary["available"].insert(magazine.available ? "true" : "false");
    }

    const std::map<std::string, std::set<std::string>>& get() const {
        return summary;
    }
};

#endif